@@ -15,6 +15,11 @@ import (
 	"testing"
 )
 
+func TestMain(m *testing.M) {
+	control.init()
+	os.Exit(m.Run())
+}
+
 func assertBool(t *testing.T, see bool, expect bool) {
 	t.Helper()
 	if see != expect {
@@ -34,7 +39,7 @@ func assertEqual(t *testing.T, a string, b string) {
 	}
 }
 
-func assertRuneEqual(t *testing.T, a rune, b rune) {
+func assertOpEqual(t *testing.T, a optype, b optype) {
 	t.Helper()
 	if a != b {
 		t.Fatalf("assertEqual: expected %q == %q", a, b)
@@ -767,7 +772,7 @@ Test to be sure we can read in a tag in inbox format.
 	t2.tagger, _ = newAttribution("")
 	t2.emailIn(msg, false)
 
-	assertEqual(t, "sample2", t2.getHumanName())
+	assertEqual(t, "sample2", t2.tagname)
 	assertEqual(t, ":2317", t2.committish)
 
 	// XXX(mem): what's undecodable?
@@ -802,7 +807,7 @@ func TestStringScan(t *testing.T) {
 
 func TestFileOp(t *testing.T) {
 	fileop1 := newFileOp(nil).construct('M', "100644", ":1", "README")
-	assertRuneEqual(t, 'M', fileop1.op)
+	assertOpEqual(t, 'M', fileop1.op)
 	assertEqual(t, "100644", fileop1.mode)
 	assertEqual(t, ":1", fileop1.ref)
 	assertEqual(t, "README", fileop1.Path)
@@ -811,7 +816,7 @@ func TestFileOp(t *testing.T) {
 	}
 
 	fileop2 := newFileOp(nil).construct('M', "100755", ":2", "DRINKME")
-	assertRuneEqual(t, 'M', fileop2.op)
+	assertOpEqual(t, 'M', fileop2.op)
 	assertEqual(t, "100755", fileop2.mode)
 	assertEqual(t, ":2", fileop2.ref)
 	assertEqual(t, "DRINKME", fileop2.Path)
@@ -820,30 +825,30 @@ func TestFileOp(t *testing.T) {
 	}
 
 	fileop3 := newFileOp(nil).construct('D', "DRINKME")
-	assertRuneEqual(t, 'D', fileop3.op)
+	assertOpEqual(t, 'D', fileop3.op)
 	assertEqual(t, "DRINKME", fileop3.Path)
 	if !fileop3.paths(nil).Equal(orderedStringSet{"DRINKME"}) {
 		t.Error("fileop3 path extraction failed equality check")
 	}
 
 	fileop4 := newFileOp(nil).construct('R', "DRINKME", "EATME")
-	assertRuneEqual(t, 'R', fileop4.op)
+	assertOpEqual(t, 'R', fileop4.op)
 	assertEqual(t, "DRINKME", fileop4.Source)
-	assertEqual(t, "EATME", fileop4.Target)
+	assertEqual(t, "EATME", fileop4.Path)
 	if !fileop4.paths(nil).Equal(orderedStringSet{"EATME", "DRINKME"}) {
 		t.Error("fileop4 path extraction failed equality check")
 	}
 
 	fileop5 := newFileOp(nil).construct('C', "DRINKME", "EATME")
-	assertRuneEqual(t, 'C', fileop5.op)
+	assertOpEqual(t, 'C', fileop5.op)
 	assertEqual(t, "DRINKME", fileop5.Source)
-	assertEqual(t, "EATME", fileop5.Target)
+	assertEqual(t, "EATME", fileop5.Path)
 	if !fileop5.paths(nil).Equal(orderedStringSet{"EATME", "DRINKME"}) {
 		t.Error("fileop5 path extraction failed equality check")
 	}
 
 	fileop6 := newFileOp(nil).construct('N', ":3", "EATME")
-	assertRuneEqual(t, 'N', fileop6.op)
+	assertOpEqual(t, 'N', fileop6.op)
 	assertEqual(t, ":3", fileop6.ref)
 	assertEqual(t, "EATME", fileop6.Path)
 	if !fileop6.paths(nil).Equal(orderedStringSet{"EATME"}) {
@@ -851,14 +856,14 @@ func TestFileOp(t *testing.T) {
 	}
 
 	fileop7 := newFileOp(nil).construct('d')
-	assertRuneEqual(t, 'd', fileop7.op)
+	assertOpEqual(t, 'd', fileop7.op)
 	if !fileop7.paths(nil).Equal(orderedStringSet{}) {
 		t.Error("fileop7 path extraction failed equality check")
 	}
 
 	line8 := "M 100644 :4 COPYING"
 	fileop8 := newFileOp(nil).parse(line8)
-	assertRuneEqual(t, 'M', fileop8.op)
+	assertOpEqual(t, 'M', fileop8.op)
 	assertEqual(t, "100644", fileop8.mode)
 	assertEqual(t, ":4", fileop8.ref)
 	assertEqual(t, "COPYING", fileop8.Path)
@@ -866,7 +871,7 @@ func TestFileOp(t *testing.T) {
 
 	line9 := "M 100755 :5 runme.sh"
 	fileop9 := newFileOp(nil).parse(line9)
-	assertRuneEqual(t, 'M', fileop9.op)
+	assertOpEqual(t, 'M', fileop9.op)
 	assertEqual(t, "100755", fileop9.mode)
 	assertEqual(t, ":5", fileop9.ref)
 	assertEqual(t, "runme.sh", fileop9.Path)
@@ -874,34 +879,34 @@ func TestFileOp(t *testing.T) {
 
 	line10 := "D deleteme"
 	fileop10 := newFileOp(nil).parse(line10)
-	assertRuneEqual(t, 'D', fileop10.op)
+	assertOpEqual(t, 'D', fileop10.op)
 	assertEqual(t, "deleteme", fileop10.Path)
 	assertEqual(t, line10+"\n", fileop10.String())
 
 	line11 := `R "DRINKME" "EATME"`
 	fileop11 := newFileOp(nil).parse(line11)
-	assertRuneEqual(t, 'R', fileop11.op)
+	assertOpEqual(t, 'R', fileop11.op)
 	assertEqual(t, "DRINKME", fileop11.Source)
-	assertEqual(t, "EATME", fileop11.Target)
+	assertEqual(t, "EATME", fileop11.Path)
 	assertEqual(t, line11+"\n", fileop11.String())
 
 	line12 := `C "DRINKME" "EATME"`
 	fileop12 := newFileOp(nil).parse(line12)
-	assertRuneEqual(t, 'C', fileop12.op)
+	assertOpEqual(t, 'C', fileop12.op)
 	assertEqual(t, "DRINKME", fileop12.Source)
-	assertEqual(t, "EATME", fileop12.Target)
+	assertEqual(t, "EATME", fileop12.Path)
 	assertEqual(t, line12+"\n", fileop12.String())
 
 	line13 := "N :6 EATME"
 	fileop13 := newFileOp(nil).parse(line13)
-	assertRuneEqual(t, 'N', fileop13.op)
+	assertOpEqual(t, 'N', fileop13.op)
 	assertEqual(t, ":6", fileop13.ref)
 	assertEqual(t, "EATME", fileop13.Path)
 	assertEqual(t, line13+"\n", fileop13.String())
 
 	line14 := "deleteall"
 	fileop14 := newFileOp(nil).parse(line14)
-	assertRuneEqual(t, 'd', fileop14.op)
+	assertOpEqual(t, 'd', fileop14.op)
 	assertEqual(t, line14+"\n", fileop14.String())
 
 	if fileop1.relevant(fileop2) {
@@ -1030,7 +1035,7 @@ func TestCommitMethods(t *testing.T) {
 
 	// Check that various reports look sane, at least matching each other
 	assertEqual(t, commit.lister(nullOrderedStringSet, 42, 0),
-		"    43 2016-03-14T23:32:27Z     :2 Example commit for unit testing")
+		"    43 2016-03-14T23:32:27Z     :2 621be4 Example commit for unit testing")
 	assertEqual(t, commit.actionStamp(),
 		"2016-03-14T23:32:27Z!esr@thyrsus.com")
 	assertEqual(t, commit.showlegacy(), "")
@@ -1060,6 +1065,28 @@ Example commit for unit testing, modified.
 	}
 }
 
+func TestCommonDirectory(t *testing.T) {
+	repo := newRepository("fubar")
+	defer repo.cleanup()
+	addop := func(commit *Commit, line string) {
+		commit.appendOperation(newFileOp(repo).parse(line))
+	}
+	commit1 := newCommit(repo)
+	addop(commit1, "M 100644 :1 foo/rat")
+	addop(commit1, "M 100644 :2 foo/bat")
+	addop(commit1, "M 100644 :3 foo/mung/bletch")
+	assertEqual(t, commit1.commonDirectory(), "foo/")
+	commit2 := newCommit(repo)
+	addop(commit2, "M 100644 :1 argle/bargle")
+	addop(commit2, "M 100644 :2 mumble/frotz")
+	assertEqual(t, commit2.commonDirectory(), "")
+	commit3 := newCommit(repo)
+	addop(commit3, "M 100644 :1 foo/bar/rat")
+	addop(commit3, "M 100644 :2 foo/bar/bat")
+	addop(commit2, "M 100644 :3 foo/bar/mung/bletch")
+	assertEqual(t, commit3.commonDirectory(), "foo/bar/")
+}
+
 func TestParentChildMethods(t *testing.T) {
 	repo := newRepository("fubar")
 	defer repo.cleanup()
@@ -1219,7 +1246,7 @@ func TestBranchbase(t *testing.T) {
 }
 
 func TestCapture(t *testing.T) {
-	data, err0 := captureFromProcess("echo stdout; echo 1>&2 stderr")
+	data, err0 := captureFromProcess("echo stdout; echo 1>&2 stderr", control.baton)
 	if err0 != nil {
 		t.Fatalf("error while spawning process: %v", err0)
 	}
@@ -1306,7 +1333,7 @@ M 100644 :3 README
 	defer repo.cleanup()
 	sp := newStreamParser(repo)
 	r := strings.NewReader(rawdump)
-	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load")
+	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load", control.baton)
 
 	assertBool(t, len(repo.events) == 4, true)
 	assertBool(t, repo.events[3].getMark() == ":4", true)
@@ -1479,15 +1506,15 @@ func TestFastImportParse2(t *testing.T) {
 	defer repo.cleanup()
 	sp := newStreamParser(repo)
 	r := strings.NewReader(rawdump)
-	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load")
+	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load", control.baton)
 
 	testTag1, ok1 := repo.events[len(repo.events)-1].(*Tag)
 	assertBool(t, ok1, true)
-	assertEqual(t, "refs/tags/with-comment", testTag1.name)
+	assertEqual(t, "with-comment", testTag1.tagname)
 
 	testTag2, ok2 := repo.events[len(repo.events)-2].(*Tag)
 	assertBool(t, ok2, true)
-	assertEqual(t, "refs/tags/no-comment", testTag2.name)
+	assertEqual(t, "no-comment", testTag2.tagname)
 
 	testReset, ok2 := repo.events[1].(*Reset)
 	assertBool(t, ok2, true)
@@ -1495,7 +1522,7 @@ func TestFastImportParse2(t *testing.T) {
 
 	// Check roundtripping via fastExport
 	var a strings.Builder
-	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil); err != nil {
+	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil, control.baton); err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
 	assertEqual(t, rawdump, a.String())
@@ -1525,7 +1552,7 @@ data 0
 `
 	a.Reset()
 	// Check partial export - Event 4 is the second commit
-	if err := repo.fastExport(newOrderedIntSet(4), &a, nullStringSet, nil); err != nil {
+	if err := repo.fastExport(newOrderedIntSet(4), &a, nullStringSet, nil, control.baton); err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
 	assertEqual(t, onecommit, a.String())
@@ -1561,7 +1588,7 @@ data 0
 
 	// Test appending a done marker
 	assertIntEqual(t, len(repo.events), 11)
-	repo.addEvent(newPassthrough(repo, "done"))
+	repo.addEvent(newPassthrough(repo, "done\n"))
 	assertIntEqual(t, len(repo.events), 12)
 
 	// Test appending passthrough to make sure it's inserted before "done"
@@ -1571,7 +1598,7 @@ data 0
 		passthrough, ok := event.(*Passthrough)
 		return ok && passthrough.text == payload
 	}
-	assertBool(t, isPassthrough(repo.events[12], "done"), true)
+	assertBool(t, isPassthrough(repo.events[12], "done\n"), true)
 	assertBool(t, isPassthrough(repo.events[11], "boogabooga"), true)
 
 	assertEqual(t, repo.earliestCommit().Comment, "First revision.\n")
@@ -1586,12 +1613,12 @@ func TestDelete(t *testing.T) {
 	defer repo.cleanup()
 	sp := newStreamParser(repo)
 	r := strings.NewReader(rawdump)
-	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load")
+	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load", control.baton)
 
 	thirdcommit := repo.markToIndex(":6")
-	repo.delete(orderedIntSet{thirdcommit}, nil)
+	repo.delete(orderedIntSet{thirdcommit}, nil, control.baton)
 	var a strings.Builder
-	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil); err != nil {
+	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil, control.baton); err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
 
@@ -1645,7 +1672,7 @@ func TestResort(t *testing.T) {
 	defer repo.cleanup()
 	sp := newStreamParser(repo)
 	r := strings.NewReader(rawdump)
-	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load")
+	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load", control.baton)
 
 	// Reverse the event array, trick from SliceTricks
 	for i := len(repo.events)/2 - 1; i >= 0; i-- {
@@ -1657,7 +1684,7 @@ func TestResort(t *testing.T) {
 	//repo.resort()
 
 	var a strings.Builder
-	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil); err != nil {
+	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil, control.baton); err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
 	//assertEqual(t, "", a.String())
@@ -1744,13 +1771,13 @@ this is a test tag
 	defer repo.cleanup()
 	sp := newStreamParser(repo)
 	r := strings.NewReader(doubled)
-	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load")
+	sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load", control.baton)
 
 	//verbose = debugUNITE
 	repo.renumber(1, nil)
 
 	var a strings.Builder
-	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil); err != nil {
+	if err := repo.fastExport(repo.all(), &a, nullStringSet, nil, control.baton); err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
 
@@ -1825,18 +1852,18 @@ func TestPathMap(t *testing.T) {
 	assertEqual(t, p.String(), "{}")
 	// Test copyFrom
 	p.set("foo/bar", 42)
-	p.copyFrom("baz/qux", p, "foo")
+	p.copyFrom("baz/qux", p, "foo", "(unexpected)")
 	_, contains = p.get("baz/qux/bar")
 	assertTrue(t, contains)
 	p.set("gronk", 0)
-	p.copyFrom("bat", p, "")
+	p.copyFrom("bat", p, "", "(unexpected)")
 	_, contains = p.get("bat/baz/qux/bar")
 	assertTrue(t, contains)
 	_, contains = p.get("bat/gronk")
 	assertTrue(t, contains)
-	p.copyFrom("", p, "foo")
+	p.copyFrom("", p, "foo", "(unexpected)")
 	_, contains = p.get("bat/gronk")
-	p.copyFrom("", p, "")
+	p.copyFrom("", p, "", "(unexpected)")
 	assertTrue(t, !contains)
 	_, contains = p.get("bat/baz/qux/bar")
 	assertTrue(t, !contains)
@@ -1851,9 +1878,10 @@ func TestDeclaredBranch(t *testing.T) {
 		path             string
 		isDeclaredBranch bool
 	}
-	var branchsets = []orderedStringSet{
-		orderedStringSet{"trunk", "tags/*", "branches/*"},
-		orderedStringSet{"trunk", "tags/*", "branches/*", "*"},
+	var branchifyOptions = []string{
+		"--branchify=trunk:tags/*:branches/*",
+		"--branchify=trunk:tags/*:branches/*:*",
+		"--branchify=trunk:tags/*/*:branches/*",
 	}
 	var testcases = [][]testcase{
 		{
@@ -1882,16 +1910,28 @@ func TestDeclaredBranch(t *testing.T) {
 			{"/", false},
 			{"", false},
 		},
+		{
+			{"trunk", true},
+			{"foobar", false},
+			{"branches/foobar", true},
+			{"branches/foobar/test", false},
+			{"tags/foobar", false},
+			{"tags/foobar/cetc", true},
+			{"tag/foobar", false},
+			{"tags", false},
+			{"branches", false},
+			{"/", false},
+			{"", false},
+		},
 	}
-	control.listOptions = make(map[string]orderedStringSet)
-	for idx, branchset := range branchsets {
-		branchset := branchset
+	for idx, branchifyOption := range branchifyOptions {
 		t.Run(fmt.Sprint(idx), func(t *testing.T) {
 			for idx, test := range testcases[idx] {
 				test := test
 				t.Run(fmt.Sprint(idx), func(t *testing.T) {
-					control.listOptions["svn_branchify"] = branchset
-					assertBool(t, isDeclaredBranch(test.path), test.isDeclaredBranch)
+					sp := new(StreamParser)
+					sp.initBranchify(newStringSet(branchifyOption))
+					assertBool(t, sp.isDeclaredBranch(test.path), test.isDeclaredBranch)
 				})
 			}
 		})
@@ -1899,8 +1939,8 @@ func TestDeclaredBranch(t *testing.T) {
 }
 
 func TestBranchSplit(t *testing.T) {
-	control.listOptions = make(map[string]orderedStringSet)
-	control.listOptions["svn_branchify"] = orderedStringSet{"trunk", "tags/*", "branches/*", "*"}
+	sp := new(StreamParser)
+	sp.initBranchify(nullStringSet)
 	type splitTestEntry struct {
 		raw    string
 		branch string
@@ -1916,7 +1956,7 @@ func TestBranchSplit(t *testing.T) {
 		{"branches/foo/bar/baz", "branches/foo", "bar/baz"},
 	}
 	for _, tst := range splitTestTable {
-		b, p := splitSVNBranchPath(tst.raw)
+		b, p := sp.splitSVNBranchPath(tst.raw)
 		assertEqual(t, b, tst.branch)
 		assertEqual(t, p, tst.path)
 	}
@@ -1997,3 +2037,224 @@ func TestWalkManifests(t *testing.T) {
 	}
 	assertTrue(t, num == 0)
 }
+
+func TestFilterRegex(t *testing.T) {
+
+	// test 'filter regex /orig/replace/[flags]'
+
+	// Repo repeats string 'o123' twice in each occurrence of
+	// filename, blob, tagger, tag-name, tag-comment, author,
+	// committer, and comment, . Replacement tested against repeat
+	// flags 0,1,2,g
+
+	rawdump := `
+blob
+mark :1
+data 14
+BLB o123-o123
+
+reset refs/heads/branch-o123-o123
+commit refs/heads/branch-o123-o123
+mark :2
+committer John Smith <com@o123-o123.com> 0 +0000
+author John Smith <aut@o123-o123.com> 0 +0000
+data 14
+CMT o123-o123
+M 100644 :1 o123-o123
+
+tag o123-o123
+from :2
+tagger John Smith <tgr@o123-o123.com> 20 +0000
+data 13
+TAG o123-o123
+
+`
+	type testcase struct {
+		safety      bool
+		shouldAbort bool   // expect script abort
+		ss          string // selection-set
+		regex       string // regex to apply
+		descr       string // test description
+		expect      []string
+	}
+
+	const safetyOff = false
+	const safetyOn = true
+	const willAbort = true
+	const isValid = false
+
+	var cases = []testcase{
+
+		// Test Cases:
+		//   c=comment
+		//   C=committer
+		//   a=author
+		//   g=replaceall
+		//   N=replaceN
+		//   backreferences
+
+		// Attempt every object, with safety on, aborts
+		{safetyOn, willAbort, "(1..$)", "/o123/s456/", "safe-dft-sub-every-aborts",
+			[]string{},
+		},
+
+		// Apply to every object, with safety off, replacing first occurrance (default)
+		{safetyOff, isValid, "(1..$)", "/o123/s456/", "unsafe-dft-first-every",
+			[]string{
+				"BLB s456-o123\n",
+				"CMT s456-o123\n", "aut@s456-o123.com", "com@s456-o123.com",
+				"TAG s456-o123", "tgr@s456-o123.com",
+			},
+		},
+
+		// Replace every occurrence in every object, safety off
+		{safetyOff, isValid, "(1..$)", "/o123/s456/g", "unsafe-sub-every",
+			[]string{
+				"BLB s456-s456\n",
+				"CMT s456-s456\n", "aut@s456-s456.com", "com@s456-s456.com",
+				"TAG s456-s456", "tgr@s456-s456.com",
+			},
+		},
+
+		// default first in commit comments
+		{safetyOn, isValid, "(1..$ & =C)", "/o123/s456/c", "dft-first-in-comment",
+			[]string{
+				"BLB o123-o123\n",
+				"CMT s456-o123\n", "aut@o123-o123.com", "com@o123-o123.com",
+				"TAG o123-o123", "tgr@o123-o123.com",
+			},
+		},
+
+		// explicit first in commit comments "c1"
+		{safetyOn, isValid, "(1..$ & =C)", "/o123/s456/c1", "explicit-first-in-comment-c1",
+			[]string{
+				"BLB o123-o123\n",
+				"CMT s456-o123\n", "aut@o123-o123.com", "com@o123-o123.com",
+				"TAG o123-o123", "tgr@o123-o123.com",
+			},
+		},
+
+		// explicit first in commit comments "1c"
+		{safetyOn, isValid, "(1..$ & =C)", "/o123/s456/1c", "explicit-first-in-comment-1c",
+			[]string{
+				"BLB o123-o123\n",
+				"CMT s456-o123\n", "aut@o123-o123.com", "com@o123-o123.com",
+				"TAG o123-o123", "tgr@o123-o123.com",
+			},
+		},
+
+		// explicit two in commit comments "2c"
+		{safetyOn, isValid, "(1..$ & =C)", "/o123/s456/2c", "two-in-comment",
+			[]string{
+				"BLB o123-o123\n",
+				"CMT s456-s456\n", "aut@o123-o123.com", "com@o123-o123.com",
+				"TAG o123-o123", "tgr@o123-o123.com",
+			},
+		},
+
+		// default first in committer
+		{safetyOn, isValid, "(1..$ & =C)", "/o123/s456/C", "dft-first-in-committer",
+			[]string{
+				"BLB o123-o123\n",
+				"CMT o123-o123\n", "aut@o123-o123.com", "com@s456-o123.com",
+				"TAG o123-o123", "tgr@o123-o123.com",
+			},
+		},
+
+		// every in committer
+		{safetyOn, isValid, "(1..$ & =C)", "/o123/s456/gc", "committer-every",
+			[]string{
+				"BLB o123-o123\n",
+				"CMT s456-s456\n", "aut@o123-o123.com", "com@o123-o123.com",
+				"TAG o123-o123", "tgr@o123-o123.com",
+			},
+		},
+
+		// back-ref everywhere
+		{safetyOff, isValid, "(1..$)", "/o(123)/r\\1/g", "unsafe-bref-every",
+			[]string{
+				"BLB r123-r123\n",
+				"CMT r123-r123\n", "aut@r123-r123.com", "com@r123-r123.com",
+				"TAG r123-r123", "tgr@r123-r123.com",
+			},
+		},
+
+		// back-ref, default comment
+		{safetyOn, isValid, "(1..$ & =C)", "/o(123)/r\\1/cg", "bref-dft-comment",
+			[]string{
+				"BLB o123-o123\n",
+				"CMT r123-r123\n", "aut@o123-o123.com", "com@o123-o123.com",
+				"TAG o123-o123", "tgr@o123-o123.com",
+			},
+		},
+
+		// back-ref, commits, comment only, only first
+		{safetyOn, isValid, "(1..$ & =C)", "/o(123)/r\\1/1c", "bref-explicit-first",
+			[]string{
+				"BLB o123-o123\n",
+				"CMT r123-o123\n", "aut@o123-o123.com", "com@o123-o123.com",
+				"TAG o123-o123", "tgr@o123-o123.com",
+			},
+		},
+	}
+
+	for idx, test := range cases {
+
+		test := test
+
+		t.Run(fmt.Sprint(idx, "-", test.descr), func(t *testing.T) {
+
+			// t.Parallel() // doesn't work
+
+			control.setAbort(false)
+
+			// create repo and read
+			repo := newRepository("test")
+			defer repo.cleanup() // needed?
+			r := strings.NewReader(rawdump)
+			sp := newStreamParser(repo)
+			sp.fastImport(context.TODO(), r, nullStringSet, "synthetic test load", control.baton)
+
+			// create surgeon, set repo and selection set
+			// control.listOptions = make(map[string]orderedStringSet)
+			rs := newReposurgeon()
+			rs.repolist = append(rs.repolist, repo)
+			rs.choose(repo)
+			rs.setSelectionSet(test.ss)
+
+			// all tests have valid --regex lines, not checking nil
+			fhook := newFilterCommand(repo, fmt.Sprint("regex ", test.regex))
+
+			rs.chosen().dataTraverse("", rs.selection, fhook.do, fhook.attributes, test.safety, true, control.baton)
+
+			// test results
+
+			if test.shouldAbort && control.getAbort() {
+				return
+			}
+
+			{ // Event 0 blob
+				ev := repo.events[0]
+				assertEqual(t, test.expect[0], ev.getComment())
+			}
+
+			{ // Event 2 commit
+				ev := repo.events[2]
+				assertTrue(t, ev.isCommit())
+				commit, _ := ev.(*Commit)
+
+				assertEqual(t, test.expect[1], commit.Comment)
+				assertEqual(t, test.expect[2], commit.authors[0].email)
+				assertEqual(t, test.expect[3], commit.committer.email)
+			}
+
+			{ // Event 4 tag
+				ev := repo.events[3]
+				tag, _ := ev.(*Tag)
+				assertEqual(t, test.expect[4], tag.Comment)
+				assertEqual(t, test.expect[5], tag.tagger.email)
+			}
+
+		})
+	}
+}
